@@ -25,8 +25,6 @@ import (
 
 	"golang.org/x/sync/errgroup"
 
-	"github.com/gohugoio/hugo/hugofs"
-
 	"log"
 	"os"
 	"path/filepath"
@@ -44,7 +42,6 @@ import (
 	"regexp"
 
 	"github.com/fsnotify/fsnotify"
-	"github.com/gohugoio/hugo/deps"
 	"github.com/gohugoio/hugo/helpers"
 	"github.com/gohugoio/hugo/hugolib"
 	"github.com/gohugoio/hugo/livereload"
@@ -55,7 +52,6 @@ import (
 	"github.com/spf13/fsync"
 	jww "github.com/spf13/jwalterweatherman"
 	"github.com/spf13/nitro"
-	"github.com/spf13/viper"
 )
 
 // Hugo represents the Hugo sites to build. This variable is exported as it
@@ -142,10 +138,6 @@ Complete documentation is available at http://gohugo.io/.`,
 			return err
 		}
 
-		if buildWatch {
-			c.watchConfig()
-		}
-
 		return c.build()
 	},
 }
@@ -301,129 +293,11 @@ func init() {
 // InitializeConfig initializes a config file with sensible default configuration flags.
 func InitializeConfig(running bool, doWithCommandeer func(c *commandeer) error, subCmdVs ...*cobra.Command) (*commandeer, error) {
 
-	var cfg *deps.DepsCfg = &deps.DepsCfg{}
-
-	// Init file systems. This may be changed at a later point.
-	osFs := hugofs.Os
-
-	config, err := hugolib.LoadConfig(osFs, source, cfgFile)
+	c, err := newCommandeer(running, doWithCommandeer, subCmdVs...)
 	if err != nil {
 		return nil, err
 	}
 
-	// Init file systems. This may be changed at a later point.
-	cfg.Cfg = config
-
-	c, err := newCommandeer(cfg, running)
-	if err != nil {
-		return nil, err
-	}
-
-	for _, cmdV := range append([]*cobra.Command{hugoCmdV}, subCmdVs...) {
-		c.initializeFlags(cmdV)
-	}
-
-	if baseURL != "" {
-		config.Set("baseURL", baseURL)
-	}
-
-	if doWithCommandeer != nil {
-		if err := doWithCommandeer(c); err != nil {
-			return nil, err
-		}
-	}
-
-	if len(disableKinds) > 0 {
-		c.Set("disableKinds", disableKinds)
-	}
-
-	logger, err := createLogger(cfg.Cfg)
-	if err != nil {
-		return nil, err
-	}
-
-	cfg.Logger = logger
-
-	config.Set("logI18nWarnings", logI18nWarnings)
-
-	if theme != "" {
-		config.Set("theme", theme)
-	}
-
-	if themesDir != "" {
-		config.Set("themesDir", themesDir)
-	}
-
-	if destination != "" {
-		config.Set("publishDir", destination)
-	}
-
-	var dir string
-	if source != "" {
-		dir, _ = filepath.Abs(source)
-	} else {
-		dir, _ = os.Getwd()
-	}
-	config.Set("workingDir", dir)
-
-	if contentDir != "" {
-		config.Set("contentDir", contentDir)
-	}
-
-	if layoutDir != "" {
-		config.Set("layoutDir", layoutDir)
-	}
-
-	if cacheDir != "" {
-		config.Set("cacheDir", cacheDir)
-	}
-
-	fs := hugofs.NewFrom(osFs, config)
-
-	// Hugo writes the output to memory instead of the disk.
-	// This is only used for benchmark testing. Cause the content is only visible
-	// in memory.
-	if config.GetBool("renderToMemory") {
-		fs.Destination = new(afero.MemMapFs)
-		// Rendering to memoryFS, publish to Root regardless of publishDir.
-		config.Set("publishDir", "/")
-	}
-
-	cacheDir = config.GetString("cacheDir")
-	if cacheDir != "" {
-		if helpers.FilePathSeparator != cacheDir[len(cacheDir)-1:] {
-			cacheDir = cacheDir + helpers.FilePathSeparator
-		}
-		isDir, err := helpers.DirExists(cacheDir, fs.Source)
-		utils.CheckErr(cfg.Logger, err)
-		if !isDir {
-			mkdir(cacheDir)
-		}
-		config.Set("cacheDir", cacheDir)
-	} else {
-		config.Set("cacheDir", helpers.GetTempDir("hugo_cache", fs.Source))
-	}
-
-	if err := c.initFs(fs); err != nil {
-		return nil, err
-	}
-
-	cfg.Logger.INFO.Println("Using config file:", config.ConfigFileUsed())
-
-	themeDir := c.PathSpec().GetThemeDir()
-	if themeDir != "" {
-		if _, err := cfg.Fs.Source.Stat(themeDir); os.IsNotExist(err) {
-			return nil, newSystemError("Unable to find theme Directory:", themeDir)
-		}
-	}
-
-	themeVersionMismatch, minVersion := c.isThemeVsHugoVersionMismatch()
-
-	if themeVersionMismatch {
-		cfg.Logger.ERROR.Printf("Current theme does not support Hugo version %s. Minimum version required is %s\n",
-			helpers.CurrentHugoVersion.ReleaseVersion(), minVersion)
-	}
-
 	return c, nil
 
 }
@@ -516,28 +390,14 @@ func (c *commandeer) setValueFromFlag(flags *flag.FlagSet, key string) {
 		if _, deprecated := deprecatedFlags[strings.ToLower(key)]; deprecated {
 			msg := fmt.Sprintf(`Set "%s = true" in your config.toml.
 If you need to set this configuration value from the command line, set it via an OS environment variable: "HUGO_%s=true hugo"`, key, strings.ToUpper(key))
-			// Remove in Hugo 0.37
-			helpers.Deprecated("hugo", "--"+key+" flag", msg, false)
+			// Remove in Hugo 0.38
+			helpers.Deprecated("hugo", "--"+key+" flag", msg, true)
 		}
 		f := flags.Lookup(key)
 		c.Set(key, f.Value.String())
 	}
 }
 
-func (c *commandeer) watchConfig() {
-	v := c.Cfg.(*viper.Viper)
-	v.WatchConfig()
-	v.OnConfigChange(func(e fsnotify.Event) {
-		c.Logger.FEEDBACK.Println("Config file changed:", e.Name)
-		// Force a full rebuild
-		utils.CheckErr(c.Logger, c.recreateAndBuildSites(true))
-		if !c.Cfg.GetBool("disableLiveReload") {
-			// Will block forever trying to write to a channel that nobody is reading if livereload isn't initialized
-			livereload.ForceRefresh()
-		}
-	})
-}
-
 func (c *commandeer) fullBuild() error {
 	var (
 		g         errgroup.Group
@@ -845,7 +705,7 @@ func (c *commandeer) getDirList() ([]string, error) {
 					c.Logger.ERROR.Printf("Cannot read symbolic link '%s', error was: %s", path, err)
 					return nil
 				}
-				linkfi, err := helpers.LstatIfOs(c.Fs.Source, link)
+				linkfi, err := helpers.LstatIfPossible(c.Fs.Source, link)
 				if err != nil {
 					c.Logger.ERROR.Printf("Cannot stat %q: %s", link, err)
 					return nil
@@ -883,9 +743,13 @@ func (c *commandeer) getDirList() ([]string, error) {
 
 	// SymbolicWalk will log anny ERRORs
 	_ = helpers.SymbolicWalk(c.Fs.Source, dataDir, regularWalker)
-	_ = helpers.SymbolicWalk(c.Fs.Source, c.PathSpec().AbsPathify(c.Cfg.GetString("contentDir")), symLinkWalker)
 	_ = helpers.SymbolicWalk(c.Fs.Source, i18nDir, regularWalker)
 	_ = helpers.SymbolicWalk(c.Fs.Source, layoutDir, regularWalker)
+
+	for _, contentDir := range c.PathSpec().ContentDirs() {
+		_ = helpers.SymbolicWalk(c.Fs.Source, contentDir.Value, symLinkWalker)
+	}
+
 	for _, staticDir := range staticDirs {
 		_ = helpers.SymbolicWalk(c.Fs.Source, staticDir, regularWalker)
 	}
@@ -942,6 +806,8 @@ func (c *commandeer) resetAndBuildSites() (err error) {
 
 func (c *commandeer) initSites() error {
 	if Hugo != nil {
+		Hugo.Cfg = c.Cfg
+		Hugo.Log.ResetLogCounters()
 		return nil
 	}
 	h, err := hugolib.NewHugoSites(*c.DepsCfg)
@@ -1008,6 +874,15 @@ func (c *commandeer) newWatcher(dirList ...string) (*watcher.Batcher, error) {
 		}
 	}
 
+	// Identifies changes to config (config.toml) files.
+	configSet := make(map[string]bool)
+
+	for _, configFile := range c.configFiles {
+		c.Logger.FEEDBACK.Println("Watching for config changes in", configFile)
+		watcher.Add(configFile)
+		configSet[configFile] = true
+	}
+
 	go func() {
 		for {
 			select {
@@ -1020,6 +895,21 @@ func (c *commandeer) newWatcher(dirList ...string) (*watcher.Batcher, error) {
 				// Special handling for symbolic links inside /content.
 				filtered := []fsnotify.Event{}
 				for _, ev := range evs {
+					if configSet[ev.Name] {
+						if ev.Op&fsnotify.Chmod == fsnotify.Chmod {
+							continue
+						}
+						// Config file changed. Need full rebuild.
+						if err := c.loadConfig(true); err != nil {
+							jww.ERROR.Println("Failed to reload config:", err)
+						} else if err := c.recreateAndBuildSites(true); err != nil {
+							jww.ERROR.Println(err)
+						} else if !buildWatch && !c.Cfg.GetBool("disableLiveReload") {
+							livereload.ForceRefresh()
+						}
+						break
+					}
+
 					// Check the most specific first, i.e. files.
 					contentMapped := Hugo.ContentChanges.GetSymbolicLinkMappings(ev.Name)
 					if len(contentMapped) > 0 {
@@ -1211,7 +1101,7 @@ func pickOneWriteOrCreatePath(events []fsnotify.Event) string {
 
 // isThemeVsHugoVersionMismatch returns whether the current Hugo version is
 // less than the theme's min_version.
-func (c *commandeer) isThemeVsHugoVersionMismatch() (mismatch bool, requiredMinVersion string) {
+func (c *commandeer) isThemeVsHugoVersionMismatch(fs afero.Fs) (mismatch bool, requiredMinVersion string) {
 	if !c.PathSpec().ThemeSet() {
 		return
 	}
@@ -1220,13 +1110,13 @@ func (c *commandeer) isThemeVsHugoVersionMismatch() (mismatch bool, requiredMinV
 
 	path := filepath.Join(themeDir, "theme.toml")
 
-	exists, err := helpers.Exists(path, c.Fs.Source)
+	exists, err := helpers.Exists(path, fs)
 
 	if err != nil || !exists {
 		return
 	}
 
-	b, err := afero.ReadFile(c.Fs.Source, path)
+	b, err := afero.ReadFile(fs, path)
 
 	tomlMeta, err := parser.HandleTOMLMetaData(b)
 
