@@ -1,4 +1,4 @@
-// Copyright 2016 The Hugo Authors. All rights reserved.
+// Copyright 2018 The Hugo Authors. All rights reserved.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -25,6 +25,7 @@ import (
 	"github.com/bep/gitmap"
 
 	"github.com/gohugoio/hugo/helpers"
+	"github.com/gohugoio/hugo/hugolib/pagemeta"
 	"github.com/gohugoio/hugo/resource"
 
 	"github.com/gohugoio/hugo/output"
@@ -140,9 +141,6 @@ type Page struct {
 	Draft     bool
 	Status    string
 
-	PublishDate time.Time
-	ExpiryDate  time.Time
-
 	// PageMeta contains page stats such as word count etc.
 	PageMeta
 
@@ -223,11 +221,12 @@ type Page struct {
 	Keywords    []string
 	Data        map[string]interface{}
 
-	Date    time.Time
-	Lastmod time.Time
+	pagemeta.PageDates
 
 	Sitemap Sitemap
-	URLPath
+	pagemeta.URLPath
+	frontMatterURL string
+
 	permalink    string
 	relPermalink string
 
@@ -264,6 +263,16 @@ type Page struct {
 	targetPathDescriptorPrototype *targetPathDescriptor
 }
 
+// Sites is a convenience method to get all the Hugo sites/languages configured.
+func (p *Page) Sites() SiteInfos {
+	infos := make(SiteInfos, len(p.s.owner.Sites))
+	for i, site := range p.s.owner.Sites {
+		infos[i] = &site.Info
+	}
+
+	return infos
+}
+
 // SearchKeywords implements the related.Document interface needed for fast page searches.
 func (p *Page) SearchKeywords(cfg related.IndexConfig) ([]related.Keyword, error) {
 
@@ -379,9 +388,9 @@ func (ps Pages) String() string {
 	return fmt.Sprintf("Pages(%d)", len(ps))
 }
 
-func (ps Pages) findPagePosByFilePath(inPath string) int {
+func (ps Pages) findPagePosByFilename(filename string) int {
 	for i, x := range ps {
-		if x.Source.Path() == inPath {
+		if x.Source.Filename() == filename {
 			return i
 		}
 	}
@@ -403,16 +412,26 @@ func (ps Pages) removeFirstIfFound(p *Page) Pages {
 	return ps
 }
 
-func (ps Pages) findFirstPagePosByFilePathPrefix(prefix string) int {
+func (ps Pages) findPagePosByFilnamePrefix(prefix string) int {
 	if prefix == "" {
 		return -1
 	}
+
+	lenDiff := -1
+	currPos := -1
+	prefixLen := len(prefix)
+
+	// Find the closest match
 	for i, x := range ps {
-		if strings.HasPrefix(x.Source.Path(), prefix) {
-			return i
+		if strings.HasPrefix(x.Source.Filename(), prefix) {
+			diff := len(x.Source.Filename()) - prefixLen
+			if lenDiff == -1 || diff < lenDiff {
+				lenDiff = diff
+				currPos = i
+			}
 		}
 	}
-	return -1
+	return currPos
 }
 
 // findPagePos Given a page, it will find the position in Pages
@@ -1115,12 +1134,51 @@ func (p *Page) update(frontmatter map[string]interface{}) error {
 	// Needed for case insensitive fetching of params values
 	helpers.ToLowerMap(frontmatter)
 
-	var modified time.Time
+	var mtime time.Time
+	if p.Source.FileInfo() != nil {
+		mtime = p.Source.FileInfo().ModTime()
+	}
+
+	var gitAuthorDate time.Time
+	if p.GitInfo != nil {
+		gitAuthorDate = p.GitInfo.AuthorDate
+	}
+
+	descriptor := &pagemeta.FrontMatterDescriptor{
+		Frontmatter:   frontmatter,
+		Params:        p.params,
+		Dates:         &p.PageDates,
+		PageURLs:      &p.URLPath,
+		BaseFilename:  p.BaseFileName(),
+		ModTime:       mtime,
+		GitAuthorDate: gitAuthorDate,
+	}
+
+	// Handle the date separately
+	// TODO(bep) we need to "do more" in this area so this can be split up and
+	// more easily tested without the Page, but the coupling is strong.
+	err := p.s.frontmatterHandler.HandleDates(descriptor)
+	if err != nil {
+		p.s.Log.ERROR.Printf("Failed to handle dates for page %q: %s", p.Path(), err)
+	}
 
-	var err error
 	var draft, published, isCJKLanguage *bool
 	for k, v := range frontmatter {
 		loki := strings.ToLower(k)
+
+		if loki == "published" { // Intentionally undocumented
+			vv, err := cast.ToBoolE(v)
+			if err == nil {
+				published = &vv
+			}
+			// published may also be a date
+			continue
+		}
+
+		if p.s.frontmatterHandler.IsDateKey(loki) {
+			continue
+		}
+
 		switch loki {
 		case "title":
 			p.title = cast.ToString(v)
@@ -1139,7 +1197,7 @@ func (p *Page) update(frontmatter map[string]interface{}) error {
 				return fmt.Errorf("Only relative URLs are supported, %v provided", url)
 			}
 			p.URLPath.URL = cast.ToString(v)
-			p.URLPath.frontMatterURL = p.URLPath.URL
+			p.frontMatterURL = p.URLPath.URL
 			p.params[loki] = p.URLPath.URL
 		case "type":
 			p.contentType = cast.ToString(v)
@@ -1150,12 +1208,6 @@ func (p *Page) update(frontmatter map[string]interface{}) error {
 		case "keywords":
 			p.Keywords = cast.ToStringSlice(v)
 			p.params[loki] = p.Keywords
-		case "date":
-			p.Date, err = cast.ToTimeE(v)
-			if err != nil {
-				p.s.Log.ERROR.Printf("Failed to parse date '%v' in page %s", v, p.File.Path())
-			}
-			p.params[loki] = p.Date
 		case "headless":
 			// For now, only the leaf bundles ("index.md") can be headless (i.e. produce no output).
 			// We may expand on this in the future, but that gets more complex pretty fast.
@@ -1163,19 +1215,6 @@ func (p *Page) update(frontmatter map[string]interface{}) error {
 				p.headless = cast.ToBool(v)
 			}
 			p.params[loki] = p.headless
-		case "lastmod":
-			p.Lastmod, err = cast.ToTimeE(v)
-			if err != nil {
-				p.s.Log.ERROR.Printf("Failed to parse lastmod '%v' in page %s", v, p.File.Path())
-			}
-		case "modified":
-			vv, err := cast.ToTimeE(v)
-			if err == nil {
-				p.params[loki] = vv
-				modified = vv
-			} else {
-				p.params[loki] = cast.ToString(v)
-			}
 		case "outputs":
 			o := cast.ToStringSlice(v)
 			if len(o) > 0 {
@@ -1190,34 +1229,9 @@ func (p *Page) update(frontmatter map[string]interface{}) error {
 				}
 
 			}
-		case "publishdate", "pubdate":
-			p.PublishDate, err = cast.ToTimeE(v)
-			if err != nil {
-				p.s.Log.ERROR.Printf("Failed to parse publishdate '%v' in page %s", v, p.File.Path())
-			}
-			p.params[loki] = p.PublishDate
-		case "expirydate", "unpublishdate":
-			p.ExpiryDate, err = cast.ToTimeE(v)
-			if err != nil {
-				p.s.Log.ERROR.Printf("Failed to parse expirydate '%v' in page %s", v, p.File.Path())
-			}
 		case "draft":
 			draft = new(bool)
 			*draft = cast.ToBool(v)
-		case "published": // Intentionally undocumented
-			vv, err := cast.ToBoolE(v)
-			if err == nil {
-				published = &vv
-			} else {
-				// Some sites use this as the publishdate
-				vv, err := cast.ToTimeE(v)
-				if err == nil {
-					p.PublishDate = vv
-					p.params[loki] = p.PublishDate
-				} else {
-					p.params[loki] = cast.ToString(v)
-				}
-			}
 		case "layout":
 			p.Layout = cast.ToString(v)
 			p.params[loki] = p.Layout
@@ -1333,32 +1347,6 @@ func (p *Page) update(frontmatter map[string]interface{}) error {
 	}
 	p.params["draft"] = p.Draft
 
-	if p.Date.IsZero() {
-		p.Date = p.PublishDate
-	}
-
-	if p.PublishDate.IsZero() {
-		p.PublishDate = p.Date
-	}
-
-	if p.Date.IsZero() && p.s.Cfg.GetBool("useModTimeAsFallback") {
-		p.Date = p.Source.FileInfo().ModTime()
-	}
-
-	if p.Lastmod.IsZero() {
-		if !modified.IsZero() {
-			p.Lastmod = modified
-		} else {
-			p.Lastmod = p.Date
-		}
-
-	}
-
-	p.params["date"] = p.Date
-	p.params["lastmod"] = p.Lastmod
-	p.params["publishdate"] = p.PublishDate
-	p.params["expirydate"] = p.ExpiryDate
-
 	if isCJKLanguage != nil {
 		p.isCJKLanguage = *isCJKLanguage
 	} else if p.s.Cfg.GetBool("hasCJKLanguage") {
@@ -1618,6 +1606,15 @@ func (p *Page) parse(reader io.Reader) error {
 		meta = map[string]interface{}{}
 	}
 
+	if p.s != nil && p.s.owner != nil {
+		gi, enabled := p.s.owner.gitInfo.forPage(p)
+		if gi != nil {
+			p.GitInfo = gi
+		} else if enabled {
+			p.s.Log.WARN.Printf("Failed to find GitInfo for page %q", p.Path())
+		}
+	}
+
 	return p.update(meta)
 }
 
@@ -1865,14 +1862,6 @@ func (p *Page) String() string {
 	return fmt.Sprintf("Page(%q)", p.title)
 }
 
-type URLPath struct {
-	URL            string
-	frontMatterURL string
-	Permalink      string
-	Slug           string
-	Section        string
-}
-
 // Scratch returns the writable context associated with this Page.
 func (p *Page) Scratch() *Scratch {
 	if p.scratch == nil {
